@@ -23,7 +23,6 @@ typedef struct {
 
     /* Sound events setup */
     GtkWidget *table, *ctree, *notebook;
-    GHashTable *entries;
 
     GtkWidget *btn_filename, *btn_play;
 
@@ -57,6 +56,8 @@ static void ui_do_revert(GtkWidget *w, SoundProps *props);
 static void ui_do_ok(GtkWidget *w, SoundProps *props);
 static void ui_do_cancel(GtkWidget *w, SoundProps *props);
 static void sound_properties_play_sound(GtkWidget *btn, SoundProps *props); 
+static void reload_esd_samples(const char *config_path);
+static void reload_all_esd_samples(void);
 
 int
 main(int argc,
@@ -64,6 +65,10 @@ main(int argc,
 {
     SoundProps *sound_properties;
     int init_ret;
+    GnomeClient *client;
+    char *init_argv[3] = {NULL, "--init-session-settings", NULL};
+
+    init_argv[0] = argv[0];
 
     bindtextdomain (PACKAGE, GNOMELOCALEDIR);
     textdomain (PACKAGE);
@@ -72,49 +77,63 @@ main(int argc,
         gnome_capplet_init ("sound-properties", VERSION,
                             argc, argv, NULL, 0, NULL);
 
-    if(init_ret == 1) {
-        GnomeClient *client;
-        int token, esdpid;
-        static const char *esd_cmdline[] = {"esd", NULL};
-        char *tmpargv[3];
+    client = gnome_master_client();
 
-        client = gnome_master_client();
-
-        if(gnome_client_get_previous_id(client)) {
-            token = gnome_startup_acquire_token("GNOME_SOUND_PROPERTY",
-                                                gnome_client_get_id(client));
-        } else {
-            token = 1;
-        }
-
-        if(!token)
-            return 0;
-
-        if(gnome_config_get_bool("/sound/system/settings/start_esd=true")) {
-            char argbuf[32];
-
-            esdpid = gnome_execute_async(NULL, 1, (char **)esd_cmdline);
-            g_snprintf(argbuf, sizeof(argbuf), "%d", esdpid);
-            tmpargv[0] = "kill";
-            tmpargv[1] = argbuf;
-            tmpargv[2] = NULL;
-            gnome_client_set_shutdown_command(client, 2, tmpargv);
-        }
-
-        gnome_client_set_restart_command(client, argc, argv);
-        gnome_client_set_clone_command(client, argc, argv);
+    if(client) {
+        gnome_client_set_restart_command(client, 2, init_argv);
         gnome_client_set_restart_style(client, GNOME_RESTART_ANYWAY);
+        gnome_client_flush(client);
+    }
 
+    switch(init_ret) {
+    case -1:
+        return 1;
+        break;
+    case 1:
+        {
+            int token, esdpid;
+            static const char *esd_cmdline[] = {"esd", NULL};
+            char *tmpargv[3];
+
+            if(client && gnome_client_get_previous_id(client)) {
+                token = gnome_startup_acquire_token("GNOME_SOUND_PROPERTIES",
+                                                    gnome_client_get_id(client));
+            } else
+                token = 1;
+            
+            if(!token)
+                return 0;
+            
+            if(gnome_config_get_bool("/sound/system/settings/start_esd=true")) {
+                char argbuf[32];
+                
+                esdpid = gnome_execute_async(NULL, 1, (char **)esd_cmdline);
+                g_snprintf(argbuf, sizeof(argbuf), "%d", esdpid);
+                tmpargv[0] = "kill";
+                tmpargv[1] = argbuf;
+                tmpargv[2] = NULL;
+
+                if(client)
+                    gnome_client_set_shutdown_command(client, 2, tmpargv);
+
+                sleep(1); /* lame... */
+                gnome_sound_init(NULL); /* Call it again so it will connect */
+                if(gnome_sound_connection < 0)
+                    g_warning("esd didn't start up!");
+                else if(gnome_config_get_bool("/sound/system/settings/event_sounds=true")) {
+                    reload_all_esd_samples();
+                }
+            }
+        }
         return 0;
+
+    default:
+        break;
     }
 
     sound_properties = sound_properties_create();
     
-#ifdef TESTING
-    gtk_main();
-#else
     capplet_gtk_main();
-#endif
 
     return 0;
 }
@@ -128,6 +147,7 @@ sound_properties_create(void)
     SoundProps *retval;
     GtkWidget *table, *vbox, *wtmp, *notebook;
     GtkWidget *frame;
+    char *path;
     char *filename;
     static const char *ctree_column_titles[] = {
         N_("Category"),
@@ -171,6 +191,7 @@ sound_properties_create(void)
             gtk_container_add (GTK_CONTAINER (alignment), pixmap);
 
             gtk_box_pack_start (GTK_BOX (vbox), alignment, FALSE, FALSE, 0);
+            g_free (filename);
     }   
 
     frame = gtk_frame_new ("Enable");
@@ -225,6 +246,7 @@ sound_properties_create(void)
 
     gtk_ctree_set_expander_style(GTK_CTREE(retval->ctree),
                                  GTK_CTREE_EXPANDER_SQUARE);
+    gtk_ctree_set_line_style(GTK_CTREE(retval->ctree), GTK_CTREE_LINES_DOTTED);
 
     sound_properties_regenerate_ctree(retval);
 
@@ -263,7 +285,9 @@ sound_properties_create(void)
                      1, 2, 1, 2,
                      GTK_EXPAND | GTK_SHRINK, 0,
                      GNOME_PAD_SMALL, GNOME_PAD_SMALL);
-
+    path = gnome_datadir_file ("sounds/");
+    gnome_file_entry_set_default_path(GNOME_FILE_ENTRY (retval->btn_filename), path);
+    g_free (path);
     gtk_signal_connect(GTK_OBJECT(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(retval->btn_filename))),
                        "changed",
                        GTK_SIGNAL_FUNC(sound_properties_event_change_file),
@@ -479,8 +503,7 @@ sound_properties_event_apply(GtkCTreeNode *node,
     if(!strcmp(cur_filename, ev->file))
         return;
 
-    ctmp = g_copy_strings("/sound/events/", ev->category, "/",
-                          ev->name, "/file");
+    ctmp = g_strconcat("/sound/events/", ev->category, "/", ev->name, "/file");
     gnome_config_set_string(ctmp, cur_filename);
     g_free(ctmp);
 }
@@ -496,6 +519,7 @@ sound_properties_apply(SoundProps *props)
     gnome_config_set_bool("/sound/system/settings/event_sounds",
                           GTK_TOGGLE_BUTTON(props->enable_sound_events)->active);
     gnome_config_sync();
+    reload_all_esd_samples();
 }
 
 static void
@@ -565,3 +589,96 @@ sound_properties_play_sound(GtkWidget *btn, SoundProps *props)
 
     g_free(ctmp2);
 }
+
+static void
+reload_esd_samples(const char *config_path)
+{
+    DIR *dirh;
+    char *category_name, *sample_name, *sample_file, *ctmp;
+    gpointer top_iter, event_iter;
+    struct dirent *dent;
+    GString *tmpstr;
+
+    dirh = opendir(config_path);
+    if(!dirh)
+        return;
+
+    tmpstr = g_string_new(NULL);
+
+    while((dent = readdir(dirh))) {
+        /* ignore no-good dir entries.
+           We ignore "gnome" because the system sounds are listed in there.
+        */
+
+        if (!strcmp(dent->d_name, ".")
+            || !strcmp(dent->d_name, ".."))
+            continue;
+
+        g_string_sprintf(tmpstr, "=%s/%s=", config_path, dent->d_name);
+
+        gnome_config_push_prefix(tmpstr->str);
+
+        category_name = dent->d_name;
+        ctmp = strstr(category_name, ".soundlist");
+        if(ctmp) *ctmp = '\0';
+
+        event_iter = gnome_config_init_iterator_sections(tmpstr->str);
+        while((event_iter = gnome_config_iterator_next(event_iter,
+                                                       &sample_name, NULL))) {
+            if(!strcmp(sample_name, "__section_info__")) {
+                g_free(sample_name);
+                continue;
+            }
+
+            g_string_sprintf(tmpstr, "%s/file", sample_name);
+            sample_file = gnome_config_get_string(tmpstr->str);
+
+            if(!sample_file || !*sample_file) {
+                g_free(sample_file);
+                g_free(sample_name);
+                continue;
+            }
+
+            if(*sample_file != '/') {
+                char *tmp = gnome_sound_file(sample_file);
+                g_free(sample_file);
+                sample_file = tmp;
+            }
+
+            if(sample_file) {
+                int sid;
+
+                g_string_sprintf(tmpstr, "%s/%s", category_name, sample_name);
+                sid = gnome_sound_sample_load(tmpstr->str, sample_file);
+
+                if(sid < 0)
+                    g_warning("Couldn't load sound file %s as sample %s",
+                              sample_file, tmpstr->str);
+            }
+            
+            g_free(sample_name);
+            g_free(sample_file);
+        }
+
+        gnome_config_pop_prefix();
+    }
+    closedir(dirh);
+
+    g_string_free(tmpstr, TRUE);
+}
+
+static void
+reload_all_esd_samples(void)
+{
+    char *val;
+    val = gnome_config_file("/sound/events");
+    if(val) {
+        reload_esd_samples(val);
+        g_free(val);
+    }
+    val = gnome_util_home_file("/sound/events");
+    if(val) {
+        reload_esd_samples(val);
+        g_free(val);
+    }
+}
